<?php
/*
Plugin Name: OBrien Media Midas Connector
Description: Provides WordPress core, theme, and plugin version information to O'Brien Media's Midas system to provide update notifications and assist in security monitoring of your website.
Version: 1.2
Plugin URI: https://obrienmedia.co.uk/obrien-media-version-monitoring
Author: O'Brien Media (Pembroke Digital Limited)
Author URI: https://obrienmedia.co.uk
*/

// Function to get the latest WordPress version from wordpress.org
function omvm_get_latest_wordpress_version() {
    $response = wp_remote_get('https://api.wordpress.org/core/version-check/1.7/');
    if (is_wp_error($response)) {
        return 'Error fetching latest version';
    }

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    if (!isset($data['offers'][0]['current'])) {
        return 'Error fetching latest version';
    }

    return $data['offers'][0]['current'];
}

// Function to get the required PHP version for the latest WordPress version
function omvm_get_required_php_version() {
    $response = wp_remote_get('https://wordpress.org/about/requirements/');
    if (is_wp_error($response)) {
        return 'Error fetching PHP requirements';
    }

    $body = wp_remote_retrieve_body($response);
    if (preg_match('/<strong>PHP<\/strong> ([0-9\.]+)/', $body, $matches)) {
        return $matches[1];
    }

    return 'Error fetching PHP requirements';
}

// Function to send version data
function omvm_send_version_data() {
    // Get site details
    $site_name = get_bloginfo('name');
    $site_url = get_bloginfo('url');
    $site_url = omvm_get_cleaned_domain_name($site_url); // Clean the domain name
    $wp_version = get_bloginfo('version');
    $latest_wp_version = omvm_get_latest_wordpress_version();
    $php_version = phpversion();
    $required_php_version = omvm_get_required_php_version();
    $php_upgrade_needed = version_compare($php_version, $required_php_version, '<');

    // Get active theme details
    $theme = wp_get_theme();
    $theme_data = array(
        'name' => $theme->get('Name'),
        'version' => $theme->get('Version'),
        'last_updated' => $theme->get('Last Modified'),
        'update_available' => false,
        'new_version' => '',
        'slug' => $theme->get_stylesheet()
    );

    // Get theme update information
    $theme_updates = get_site_transient('update_themes');
    if (isset($theme_updates->response[$theme->get_stylesheet()])) {
        $theme_data['update_available'] = true;
        $theme_data['new_version'] = $theme_updates->response[$theme->get_stylesheet()]['new_version'];
    }

    // Get active plugins details
    $plugins = get_option('active_plugins');
    $plugin_data = array();

    // Get plugin update information
    $plugin_updates = get_site_transient('update_plugins');


		if( ! function_exists('get_plugin_data') ){
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}
        foreach ($plugins as $plugin) {
            $plugin_info = get_plugin_data(WP_PLUGIN_DIR . '/' . $plugin);
            $plugin_last_updated = filemtime(WP_PLUGIN_DIR . '/' . $plugin);

            $update_available = false;
            $new_version = '';

            if (isset($plugin_updates->response[$plugin])) {
                $update_available = true;
                $new_version = $plugin_updates->response[$plugin]->new_version;
            }

            $plugin_data[] = array(
                'name' => $plugin_info['Name'],
                'version' => $plugin_info['Version'],
                'last_updated' => date('Y-m-d H:i:s', $plugin_last_updated),
                'update_available' => $update_available,
                'new_version' => $new_version,
                'slug' => dirname($plugin),
                'php_compatibility' => omvm_check_plugin_php_compatibility($plugin, $php_version, $required_php_version)
            );
        }


    // Prepare data for sending
    $data = array(
        'site_name' => $site_name,
        'site_url' => $site_url,
        'wp_version' => $wp_version,
        'latest_wp_version' => $latest_wp_version,
        'php_version' => $php_version,
        'php_upgrade_needed' => $php_upgrade_needed,
        'theme' => $theme_data,
        'plugins' => $plugin_data
    );

    // Convert data to JSON
    $json_data = json_encode($data);

    // URL to send data to
    $url = 'https://midas.obrienmedia.uk/api/wordpress-versions.php';

    // Send data using cURL
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $json_data);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
    $response = curl_exec($ch);
    curl_close($ch);

    // Log the response for debugging
    if ($response === FALSE) {
        error_log('Error sending data: ' . curl_error($ch));
    } else {
        error_log('Data sent successfully: ' . $response);
    }
}

// Function to check plugin PHP compatibility
function omvm_check_plugin_php_compatibility($plugin, $current_php_version, $required_php_version) {
    $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/' . $plugin);
    $requires_php = isset($plugin_data['RequiresPHP']) ? $plugin_data['RequiresPHP'] : $required_php_version;

    if (version_compare($current_php_version, $requires_php, '<')) {
        return array(
            'compatible' => false,
            'required_version' => $requires_php
        );
    }

    return array(
        'compatible' => true,
        'required_version' => $requires_php
    );
}

// Function to clean domain name
function omvm_get_cleaned_domain_name($url) {
    $parsed_url = parse_url($url, PHP_URL_HOST);
    if (substr($parsed_url, 0, 4) == "www.") {
        $parsed_url = substr($parsed_url, 4);
    }
    return $parsed_url;
}

// Hook into WordPress initialization
add_action('init', 'omvm_schedule_cron_jobs');

// Schedule the cron jobs if they are not already scheduled
function omvm_schedule_cron_jobs() {
    if (!wp_next_scheduled('omvm_send_version_data_0800')) {
        wp_schedule_event(strtotime('08:00:00'), 'daily', 'omvm_send_version_data_0800');
    }
    if (!wp_next_scheduled('omvm_send_version_data_1300')) {
        wp_schedule_event(strtotime('13:00:00'), 'daily', 'omvm_send_version_data_1300');
    }
    if (!wp_next_scheduled('omvm_send_version_data_1600')) {
        wp_schedule_event(strtotime('16:00:00'), 'daily', 'omvm_send_version_data_1600');
    }
    if (!wp_next_scheduled('omvm_send_version_data_2000')) {
        wp_schedule_event(strtotime('20:00:00'), 'daily', 'omvm_send_version_data_2000');
    }
}

// Hook our function to the cron event
add_action('omvm_send_version_data_0800', 'omvm_send_version_data');
add_action('omvm_send_version_data_1300', 'omvm_send_version_data');
add_action('omvm_send_version_data_1600', 'omvm_send_version_data');
add_action('omvm_send_version_data_2000', 'omvm_send_version_data');

// Hook into the login page access
add_action('login_init', 'omvm_send_version_data');

// Clear the scheduled events upon plugin deactivation
register_deactivation_hook(__FILE__, 'omvm_deactivate');
function omvm_deactivate() {
    $timestamp = wp_next_scheduled('omvm_send_version_data_0800');
    if ($timestamp) {
        wp_unschedule_event($timestamp, 'omvm_send_version_data_0800');
    }
    $timestamp = wp_next_scheduled('omvm_send_version_data_1300');
    if ($timestamp) {
        wp_unschedule_event($timestamp, 'omvm_send_version_data_1300');
    }
    $timestamp = wp_next_scheduled('omvm_send_version_data_1600');
    if ($timestamp) {
        wp_unschedule_event($timestamp, 'omvm_send_version_data_1600');
    }
    $timestamp = wp_next_scheduled('omvm_send_version_data_2000');
    if ($timestamp) {
        wp_unschedule_event($timestamp, 'omvm_send_version_data_2000');
    }
}

// Automatically activate the plugin upon file placement
register_activation_hook(__FILE__, 'omvm_activate_plugin');
function omvm_activate_plugin() {
    omvm_schedule_cron_jobs();
}

// Auto-activate the plugin upon file placement
if (function_exists('activate_plugin')) {
    activate_plugin(plugin_basename(__FILE__));
}
?>
